

var tickTimer = 0, fruits = [], fruitsPool, mergeFruits = [],
droppedCount = 0, lastFruitId = 6, dropping = null, fruitDropX = 0, nextDropDelay = 0,
lastDropped = null, fruitHitLimit = false, limitTimer = 0, limitWait = 0,
score = 0, highscores = [],

FRUIT_BLUEBERRY = 0, FRUIT_RASPBERRY = 1, FRUIT_STRAWBERRY = 2, FRUIT_LIME = 3, FRUIT_KIWI = 4, FRUIT_APPLE = 5, FRUIT_BANANA = 6, FRUIT_MANGO = 7, FRUIT_WATERMELON = 8, FRUIT_PINEAPPLE = 9,

//fruit data definitions
FRUIT_DEFS = [new FruitDef(67,1820,168,151, 0.00065,0.002, 0.99995,0.4,0.03, 32,64,102),//blueberry
new FruitDef(352,1782,219,192, -0.0012,0, 0.9999999,0.15,0.5, 170,19,28),//raspberry
new FruitDef(684,1742,267,262, 0,0.005, 0.999999,0.06,2.1, 155,9,7),//strawberry
new FruitDef(1079,1815,395,184, 0.001,0.01, 0.999999,0.05,2.6, 179,207,88),//lime
new FruitDef(1538,1623,457,392, -0.0014,-0.0015, 0.999995,0.01,5, 87,139,48),//kiwi
new FruitDef(55,1170,519,497, -0.0055,-0.004, 0.999999,0.06,9, 195,8,11),//apple
new FruitDef(991,1011,415,729, -0.02,0, 1,0.05,13, 243,227,94),//banana
new FruitDef(1471,925,527,613, 0.005,-0.005, 0.99995,0.04,16, 230,111,30),//mango
new FruitDef(1115,43,885,783, 0.07,0, 0.99999,0.01,25, 240,88,87),//watermelon
new FruitDef(37,59,1017,959, 0,0, 1.01,0.02,30, 248,227,106)//pineapple
],

TICKS_PER_SECOND = 64, TICK_STEP = 1/TICKS_PER_SECOND;


function InitializeGame() {
	fruitsPool = new ObjectPool(Fruit);
}


//update game
function Update() {
	//pause click
	if (dropFruit && fruitDragY/window.innerHeight > 0.85) {
		StopAudio();
		appStatus = APPSTATUS_PAUSED;
		
		EV(gamePanel,false);
		EV(pausePanel,true);
		return;
	}

	//fruit drag in world position
	fruitDropX = (fruitDragX*2/window.innerWidth-1)*aspectX;

	//physics/logic at fixed ticks per second
	tickTimer = Math.min(tickTimer+deltaTime, 1);
	while (tickTimer >= TICK_STEP) {
		//fruit dragging and dropping
		dropping.SetPos(fruitDropX, dropping.y, dropping.angle);
		nextDropDelay += TICK_STEP;
		if (nextDropDelay > 0.5) {
			lastDropped = null;
			if (dropFruit) {
				score++;
				scoreChanged = true;
				droppedCount++;
				
				lastDropped = dropping;
				dropping.arrayId = fruits.length;
				fruits.push(dropping);
				NewFruit();
				
				dropFruit = false;
				nextDropDelay = 0;
			}
		}
	
		//physics step
		UpdatePhysics();
		
		//merge fruits
		var mflen = mergeFruits.length;
		if (mflen) {
			for (var i = 0; i < mflen; i++) {
				var mfr = mergeFruits[i],
					mhit = mfr.mergeWith;
				if (combineEffectOn) PlayCombineAnim(mfr,mhit);
				
				//destroy old fruit and load next
				mfr.mergeWith = mhit.mergeWith = null;
				mhit.Destroy();
				mfr.LoadID((mfr.fruitId+1)%10);
				
				//add score and play combine sfx
				score += 5;
				scoreChanged = true;
				if (playingSfx !== SFX_COMBINE) PlaySfx(SFX_COMBINE);
			}
			mergeFruits.length = 0;
		}
		
		//detect game over limit
		if (fruitHitLimit) {
			limitTimer = 1;
			fruitHitLimit = false;
		}
		if (limitTimer > 0) {
			limitWait += TICK_STEP;
			if (limitWait > 1.5) {
				//game over
				GameOver();
				return;
			}
			
			limitTimer -= TICK_STEP;
			if (limitTimer < 0) limitWait = 0;
		}
		
		tickTimer -= TICK_STEP;
	}
	
	//update audio
	UpdateAudio();
	
	//update score
	if (scoreChanged) {
		scoreText.textContent = ""+score;
		scoreChanged = true;
	}
}


//start new game
function StartGame() {
	scoreChanged = true;
	if (music.volume > 0.01) music.play();
	NewFruit();
	appStatus = APPSTATUS_GAME;
}


//fade in game over screen
function GameOver() {
	//stop game
	StopAudio();
	fruitsPool.Return(dropping);
	lastDropped = dropping = null;
	droppedCount = limitTimer = limitWait = nextDropDelay = gameOverTimer = 0;
	lastFruitId = 6;
	
	//detect new highscore
	var hId, hslen = highscores.length;
	for (hId = 0; hId < hslen && !(score > highscores[hId]); hId++) {}
	
	if (hId < 5) {
		//update highscores
		for (var i = Math.min(5,hslen+1); i > hId; i--) highscores[i-1] = highscores[i-2];
		highscores[hId] = score;
		SaveState();
		UpdateHighScores();
	}
	endScore.innerHTML = "Score "+score+(hId < 5 ? "</br>New #"+(hId+1)+" High Score!" : "");

	appStatus = APPSTATUS_GAMEOVER;
	EV(gamePanel,false);
	EV(gameOverPanel,true);
}


//delete all fruits
function ClearFruits() {
	for (var i = 0; i < fruits.length; i++) fruitsPool.Return(fruits[i]);
	fruits.length = 0;
	score = 0;
}


//init and new fruit to drop
function NewFruit() {
	//spawn fruits farthest away in series from each other
	var spawnId = (lastFruitId+Math.floor(Math.random()*3)+4)%10;
	lastFruitId = spawnId;
	//Math.max(0, droppedCount-1)%10;
	//Math.floor(Math.pow(Math.random(),3)*10);
	//Math.max(0,droppedCount+Math.floor(Math.random()*2))%10;
	
	dropping = fruitsPool.Get();
	dropping.LoadID(spawnId);
	dropping.SetPos(fruitDropX, 0.9, Math.random()*Math.PI*2);
	dropping.xVel = dropping.angVel = 0;
	dropping.yVel = -0.5;
}


//fruit game object/rigid body
/**@constructor*/
function Fruit() {
	this.drawX = 0;
	this.drawY = 0;
	this.x = 0;
	this.y = 0;
	this.angle = 0;
	
	this.newX = 0;
	this.newY = 0;
	this.newCos = 0;
	this.newSin = 0;
	this.oldXVel = 0;
	this.oldYVel = 0;
	this.oldAngVel = 0;
	
	this.xVel = 0;
	this.yVel = 0;
	this.angVel = 0;

	this.minRepX = 0;
	this.maxRepX = 0;
	this.repelX = 0;
	this.minRepY = 0;
	this.maxRepY = 0;
	this.repelY = 0;
	
	this.collisions = [];
	this.totalCollisions = 0;
	this.energyChain = null;
	this.mergeWith = null;
	
	this.collider = null;
	this.thickness = 0;
	this.friction = 1;
	this.bounce = 1;
	this.mass = 1;
	this.boundsRadius = 0;
	
	this.fruitId = 0;
	this.arrayId = 0;
}

//load fruit by id
Fruit.prototype.LoadID = function(id) {
	var fd = FRUIT_DEFS[id];
	this.fruitId = id;
	this.collider = FRUIT_SHAPES[id];
	this.thickness = FRUIT_THICKNESS[id];
	this.friction = fd.friction;
	this.bounce = fd.bounce;
	this.mass = fd.mass;
	this.boundsRadius = (this.collider ? this.collider.boundsRadius : 0) + this.thickness;
}

//set fruit to position
Fruit.prototype.SetPos = function(x,y,ang) {
	this.x = this.drawX = x;
	this.y = this.drawY = y;
	this.angle = ang;
}

//destroy fruit
Fruit.prototype.Destroy = function() {
	var ind = this.arrayId,
		last = fruits.length-1;
	if (ind !== last) {
		var swap = fruits[last];
		swap.arrayId = ind;
		fruits[ind] = swap;
	}
	fruits.length = last;
	fruitsPool.Return(this);
}



//fruit definition
/**@constructor*/
function FruitDef(spx,spy,spw,sph,spcx,spcy, fric,bounc,mass, colRed,colGreen,colBlue) {
	this.spriteX = spx/2048;
	this.spriteY = 1-spy/2048;
	this.spriteWidth = spw/2048;
	this.spriteHeight = sph/2048;
	this.spriteCenterX = spcx;
	this.spriteCenterY = spcy;
	
	var S2W = 0.8;
	this.width = this.spriteWidth*S2W;
	this.height = this.spriteHeight*S2W;
	
	this.friction = fric;
	this.bounce = bounc;
	this.mass = mass;
	
	this.red = colRed/255;
	this.green = colGreen/255;
	this.blue = colBlue/255;
}



/*Class: ObjectPool
Object pool, inherits from Array.

Constructor: ObjectPool
Constructs new ObjectPool.

Parameters:
*function* objClass - Object class constructor.*/
/**@constructor*/
function ObjectPool(objClass) {
	Array.call(this);
	this.objectClass = objClass;
}
ObjectPool.prototype = Object.create(Array.prototype);
/*Function: Get
Get object from pool.

Returns:
*Object**/
ObjectPool.prototype.Get = function() {
	var l = this.length;
	if (l) {
		l--;
		var o = this[l];
		this.length = l;
		return o;
	} else {
		return new this.objectClass();
	}
}
/*Function: Return
Return object to pool.

Paramters:
*Object* o*/
ObjectPool.prototype.Return = function(o) {
	this.push(o);
}
