
var bgShader, bgUni, spriteShader, spriteTexUni, combineShader, combineUni,
fruitsTexture, fruitsMesh, fruitsVerts, fruitsIndices,
aspectX = 1, aspectY = 1, graphicsSeedX = 0, graphicsSeedY = 0,
gameOverTimer = 0,
combines = [], combinePool, combineData = new Float32Array(16), combineEffectOn = 1;


//initialize rendering systems
function InitializeRendering() {

	//random seed for procedural generated art
	graphicsSeedX = Math.random()*1000;
	graphicsSeedY = Math.random()*1000;
	
	//render fruits sprite sheet via procedural shader
	var fruitsShader = LoadShaders("FruitSheetV.glsl", "FruitSheet.glsl"),
		fruitsRT = new RenderTexture(2048,2048);
	SetTextureOptions(true,true,false,false);
	
	/*export fruit sprite sheet
	renderingCanvas.width = renderingCanvas.height = display.width = display.height = 2048;
	display.Bind();
	fullscreenTriangle.Bind();
	fruitsShader.Bind();
	gl.uniform2f(fruitsShader["SEED"], graphicsSeedX,graphicsSeedY);
	gl.drawArrays(gl.TRIANGLES,0,3);
	mainPanel = null;
	return;
	*/
	
	fruitsRT.Bind();
	fullscreenTriangle.Bind();
	fruitsShader.Bind();
	gl.uniform2f(fruitsShader["SEED"], graphicsSeedX,graphicsSeedY);
	gl.drawArrays(gl.TRIANGLES,0,3);
	fruitsShader.Delete();
	gl.deleteFramebuffer(fruitsRT.framebuffer);
	fruitsTexture = fruitsRT.texture;
	gl.generateMipmap(gl.TEXTURE_2D);

	//load shaders
	bgShader = LoadShaders("BGV.glsl", "BG.glsl");
	bgUni = bgShader["params"];
	
	spriteShader = LoadShaders("SpriteV.glsl","Sprite.glsl");
	spriteTexUni = spriteShader["tex"];
	
	combineShader = LoadShaders("CombineV.glsl", "Combine.glsl");
	combineUni = combineShader["params"];
	
	//fruit sprite mesh
	fruitsVerts = new Float32Array(4);
	fruitsIndices = new Uint16Array(6);
	fruitsMesh = new Mesh([new MeshAttrib("vPos",4,4,gl.FLOAT)]);
	
	//combine animations
	combinePool = new ObjectPool(CombineAnimation);
	
	
	//window display resizing
	OnWindowResize();
	window.addEventListener("resize", OnWindowResize);
}


//on window resize event
function OnWindowResize() {
	SetDisplaySize(window.innerWidth,window.innerHeight,true);
	
	//resize aspect to fit game field
	aspectX = display.width*1.04/display.height;
	aspectY = 1.04;
	
	var ratio = aspectX/0.64;
	if (ratio < 1) {
		aspectX = 0.64;
		aspectY /= ratio;
	}
}


//render
function Render() {
	display.Bind();
	
	//render background
	fullscreenTriangle.Bind();
	bgShader.Bind();
	var limit;
	if (appStatus === APPSTATUS_GAMEOVER) {
		limit = 1+gameOverTimer;
		gameOverTimer += deltaTime;
	} else limit = (limitTimer > 0 ? (limitWait*2)%1 : 0);
	gl.uniform4f(bgUni, aspectX,aspectY,graphicsSeedX, limit);
	gl.drawArrays(gl.TRIANGLES,0,3);
	
	//draw fruit sprites
	var nfruits = fruits.length + (dropping ? 1 : 0);
	gl.enable(gl.BLEND);
	if (nfruits) {
		fruitsMesh.Bind();
	
		var nverts = nfruits*16;
		if (nverts > fruitsVerts.length) {
			var vlen = fruitsVerts.length, ilen = fruitsIndices.length;
			while (vlen < nverts) {
				vlen *= 2;
				ilen *= 2;
			}
			fruitsVerts = new Float32Array(vlen);
			fruitsIndices = new Uint16Array(ilen);
			
			//initialize indices once since they stay constant
			var lut = [2,1,0, 3,1,2];
			for (var i = 0; i < ilen; i++) {
				var fi = Math.floor(i/6)*4;
				fruitsIndices[i] = fi+lut[i%6];
			}
			fruitsMesh.SetIndices(fruitsIndices);
		}
		
		//build mesh vertices
		var vi = 0, fv = fruitsVerts, asx = 1/aspectX, asy = 1/aspectY;
		for (var i = 0; i < nfruits; i++) {
			var fr = (i === fruits.length ? dropping : fruits[i]), fd = FRUIT_DEFS[fr.fruitId],
				frx = fr.drawX, fry = fr.drawY,
				frang = fr.angle, acs = Math.cos(frang), asn = Math.sin(frang),
				spcX = -fd.spriteCenterX, spcY = -fd.spriteCenterY;
				
			//top left
			fv[vi++] = (frx + (-fd.width+spcX)*acs + (fd.height+spcY)*asn)*asx;
			fv[vi++] = (fry + (-fd.width+spcX)*-asn + (fd.height+spcY)*acs)*asy;
			fv[vi++] = fd.spriteX;
			fv[vi++] = fd.spriteY;
			
			//top right
			fv[vi++] = (frx + (fd.width+spcX)*acs + (fd.height+spcY)*asn)*asx;
			fv[vi++] = (fry + (fd.width+spcX)*-asn + (fd.height+spcY)*acs)*asy;
			fv[vi++] = fd.spriteX+fd.spriteWidth;
			fv[vi++] = fd.spriteY;
			
			//bottom left
			fv[vi++] = (frx + (-fd.width+spcX)*acs + (-fd.height+spcY)*asn)*asx;
			fv[vi++] = (fry + (-fd.width+spcX)*-asn + (-fd.height+spcY)*acs)*asy;
			fv[vi++] = fd.spriteX;
			fv[vi++] = fd.spriteY-fd.spriteHeight;

			//bottom right
			fv[vi++] = (frx + (fd.width+spcX)*acs + (-fd.height+spcY)*asn)*asx;
			fv[vi++] = (fry + (fd.width+spcX)*-asn + (-fd.height+spcY)*acs)*asy;
			fv[vi++] = fd.spriteX+fd.spriteWidth;
			fv[vi++] = fd.spriteY-fd.spriteHeight;
		}
		fruitsMesh.Set(fruitsVerts.subarray(0,vi), gl.DYNAMIC_DRAW);
	
		spriteShader.Bind();
		BindTexture(spriteShader["tex"], fruitsTexture);
		gl.drawElements(gl.TRIANGLES, nfruits*6, gl.UNSIGNED_SHORT, 0);
		UnbindAllTextures();
	}
	
	//draw combine animations
	var combineLen = combines.length;
	if (combineLen) {
		fullscreenQuad.Bind();
		combineShader.Bind();
		
		var cd = combineData;
		cd[14] = 1/aspectX; cd[15] = 1/aspectY;
		for (var i = 0; i < combineLen; i++) {
			var anim = combines[i];
			cd[0] = anim.aR; cd[1] = anim.aG; cd[2] = anim.aB; cd[3] = anim.aRadius;
			cd[4] = anim.bR; cd[5] = anim.bG; cd[6] = anim.bB; cd[7] = anim.bRadius;
			cd[8] = anim.aX; cd[9] = anim.aY;
			cd[10] = anim.bX; cd[11] = anim.bY;
			cd[12] = anim.time; cd[13] = anim.seed;
			anim.time += deltaTime;
			
			gl.uniform4fv(combineUni, cd);
			gl.drawArrays(gl.TRIANGLE_STRIP, 0,4);
			
			if (anim.time >= 1) {
				//finished animation, delete
				combinePool.Return(anim);
				combineLen--;
				if (i !== combineLen) combines[i] = combines[combineLen];
				i--;
				combines.length = combineLen;
			}
		}
	}

	gl.disable(gl.BLEND);
}



/**@constructor*/
function CombineAnimation() {
	this.aX = 0;
	this.aY = 0;
	this.aRadius = 0;
	this.aR = 0;
	this.aG = 0;
	this.aB = 0;
	
	this.bX = 0;
	this.bY = 0;
	this.bRadius = 0;
	this.bR = 0;
	this.bG = 0;
	this.bB = 0;
	
	this.seed = 0;
	this.time = 0;
}

//start animation for fruits merging
function PlayCombineAnim(mfr, mhit) {
	//setup combine animation
	var anim = combinePool.Get(),
		nextId = (mfr.fruitId+1)%10,
		fd = FRUIT_DEFS[nextId],
		collider = FRUIT_SHAPES[nextId];
	
	anim.aX = mfr.x;
	anim.aY = mfr.y;
	anim.aRadius = (collider ? collider.boundsRadius : 0) + FRUIT_THICKNESS[nextId];
	anim.aR = fd.red; anim.aG = fd.green; anim.aB = fd.blue;
	
	fd = FRUIT_DEFS[mfr.fruitId];
	anim.bX = mhit.x;
	anim.bY = mhit.y;
	anim.bRadius = mhit.boundsRadius;
	anim.bR = fd.red; anim.bG = fd.green; anim.bB = fd.blue;
	
	anim.seed = Math.random()*1000;
	anim.time = 0;
	combines.push(anim);
}
