<script type="text/plain" id="BGV.glsl">
//background vertex shader
attribute vec2 vPos;
uniform vec4 params;
varying vec2 uv;

void main() {
	uv = vPos*params.xy;
	gl_Position = vec4(vPos,0,1);
}
</script>

<script type="text/plain" id="BG.glsl">
//background frag shader
uniform vec4 params;//xy = aspect, z = seed, w = height limit line
varying vec2 uv;


void main() {
	vec3 c = vec3(.4,.5,.6)+vec3(.3,.05,.2)*length(uv-vec2(0,-.7));//gradient
	c = mix(c, vec3(.9,.8,.6), clamp(uv.y,0.,1.));
	
	float gdst = length(max(abs(uv)-vec2(.6,1.),0.)),//glass
		msk = clamp(1.-gdst*20.,0.,1.);
	c *= 1.-msk*.2;
	c += msk*.2*pow(max(0.,cos(params.z+uv.x*-5.+uv.y*3.+cos(uv.y*2.)+gdst*200.)),2.);//tint
	
	if (params.w > 0.) {
		vec2 luv = abs(vec2(mod(uv.x+10.,.2)-.1,uv.y-1.))-vec2(.02,.05);
		luv.y -= max(0.,params.w-1.);
		c += vec3(1,-0.6,-0.6)*(1./(1.+10.*length(max(luv,0.))))*min(1.,params.w);//height limit checker line
	}
	
	gl_FragColor = vec4(c,1);
}
</script>



<script type="text/plain" id="CombineV.glsl">
//combine vertex shader
attribute vec2 vPos;
varying vec2 uv;
uniform vec4 params[4];

void main() {
	vec2 minp = min(params[2].xy-params[0].w, params[2].zw-params[1].w)-.2,
		maxp = max(params[2].xy+params[0].w, params[2].zw+params[1].w)+.2,
		corner = vec2(vPos.x > 0. ? maxp.x : minp.x, vPos.y > 0. ? maxp.y : minp.y);
	uv = corner;
	gl_Position = vec4(corner*params[3].zw,0,1);
}
</script>

<script type="text/plain" id="Combine.glsl">
//combine animation frag shader
varying vec2 uv;
uniform vec4 params[4];
/*0.xyz = A color, 0.w = A radius
1.xyz = B color, 1.w = B radius
2.xy = A pos, 2.zw = B pos
3.x = animation time, 3.y = seed, 3.zw = aspect*/

void main() {
	vec2 a = uv-params[2].xy;
	float aLen = length(a), aAng = atan(a.y,a.x);
	vec2 b = uv-params[2].zw+(a/max(1e-6,aLen))*params[3].x/(1.+aLen*(5.-params[3].x*5.));
	float bLen = length(b), bAng = atan(b.y,b.x),
		aDst = aLen-params[0].w+cos(aLen*50.+aAng*20.+params[3].x*12.+params[3].y)*.02,
		bDst = bLen-params[1].w+cos(bLen*50.+bAng*20.+params[3].x*12.+params[3].y*3.37624)*.02;
	
	vec4 c = vec4(0);
	c += vec4(params[0].xyz,1) * .5 * clamp(1.-aDst*6.,0.,1.);
	c += vec4(params[1].xyz,.5) * clamp(1.-bDst*6.,0.,1.);
	c *= min(1., params[3].x*10.)-max(0.,(params[3].x-.1)*(1./.9));
	
	gl_FragColor = c;
}
</script>



<script type="text/plain" id="FruitSheetV.glsl">
//fruit sprite sheet vertex shader
attribute vec2 vPos;
varying vec2 uv;

void main() {
	uv = vPos*2.+2.;
	gl_Position = vec4(vPos,0,1);
}
</script>

<script type="text/plain" id="FruitSheet.glsl">
//fruit sprite sheet frag shader
varying vec2 uv;
uniform vec2 SEED;


#define PI 3.141592653589793

#define AAS (2048.*.2)

#define OUTLINE(c,d) vec4(c*clamp(-d*AAS,0.,1.), clamp(1.-(d-.02)*AAS,0.,1.))

#define BLEND(ac,bc) vec4(mix(ac.xyz,bc.xyz, bc.w), max(ac.w,bc.w))


float Tri(float x) {
	return abs(fract(x)-.5)*2.;
}

mat2 Rot(float ang) {
	float cs = cos(ang), sn = sin(ang);
	return mat2(cs,sn,-sn,cs);
}

float SmoothMin(float a, float b, float range) {
	float am = clamp(.5 + .5*(a-b)/range, 0.,1.);
	return mix(a,b,am) - range*am*(1.-am);
}


vec4 Blueberry(vec2 p) {
	if (length(p) > .15) return vec4(0);
	
	vec2 tp = p;
	
	p += cos(SEED/13.+p.yx*(40.+mod(SEED,15.)))*.0015;
	p.x = p.x-clamp(p.x,-.02,.02);
	float d = length(p)-.08;
	
	tp = fract((tp+10.+cos(tp.yx*8.+fract(SEED*.01278643))*.4)*5.)*2.-1.;
	float cl = length(tp)/length(vec2(1));
	vec3 c = mix(vec3(13,36,69)/255., vec3(42,79,120)/255., cl);
	return OUTLINE(c, d);
}


vec4 Raspberry(vec2 p) {
	if (length(p) > .2) return vec4(0);
	
	vec2 cp = p;
	
	float ang = atan(p.x,p.y);
	p.y += max(0.,1.-abs(ang))*.05;
	float d = length(p)-.14;
	d += (.5+.5*cos(ang*(14.+mod(SEED.x,5.))))*.005;
	
	cp = (cp+cos(SEED*.01+cp.yx*(50.+mod(SEED*.128695,10.)))*.01+10.)*27.;
	cp.x += mod(floor(cp.y),2.)*.5;
	float cl = length(fract(cp)*2.-1.)/length(vec2(1));
	vec3 c = mix(vec3(218,2,12)/255.*(1.-cl*.5), vec3(254,169,197)/255., pow((cos(3.+cp.y*6.+cos(cp.x*128.)*.1)*.5+.5)*cl,2.));
	return OUTLINE(c, d);
}


vec4 Strawberry(vec2 p) {
	if (length(p) > .25) return vec4(0);
	
	p += cos(SEED.yx*.01+p.yx*(25.+mod(SEED*.0838764,15.)))*.003;
	
	//leaves
	vec2 lp = p-vec2(0,.15);
	float lang = 100.+atan(lp.x,lp.y)/PI*(3.+mod(SEED.x*.333333,3.)),
		ld = length(lp*vec2(.6,1))+Tri(lang)*.02-.05;
	vec3 ls = vec3(118,189,27)/255.*(.75+fract(lang)*lp.y*20.*.5);
	vec4 lc = OUTLINE(ls, ld);
	
	//berry
	float ang = mod(atan(p.y,p.x)+PI*2.+1.6, PI*2./3.)+PI*.5/3.;
	vec2 ap = vec2(cos(ang),sin(ang))*length(p);
	float d = length(ap-vec2(0,-.1))-.27;
	
	vec2 sp = (p+1.)/(.014+mod(SEED.x*.011111,.005));
	sp.x += mod(floor(sp.y/2.),2.);
	sp = mod(sp,2.)-1.;
	sp.y *= .35;
	float lsp = length(sp);
	vec3 bs = mix(vec3(206,9,30)/255.*min(1.,mod(SEED.y*.052387,.5)+lsp*2.)+(p.x*(mod(SEED.y*.07376,2.)-1.)+p.y-.1), vec3(241,180,85)/255., clamp(1.1-lsp*4., 0.,1.));
	vec4 bc = OUTLINE(bs, d);
	
	return BLEND(bc,lc);
}


vec4 Lime(vec2 p) {
	if (length(p) > .35) return vec4(0);
	
	float cl = length(p),
		d = max(cl-.3, p.y+.06);
		
	vec2 tp = vec2(atan(p.x,p.y)/PI,cl);
	tp = fract((tp+10.+cos(tp.yx*58.+4.*fract(SEED*.1278643))*.01)*25.)*2.-1.;
	tp = abs(tp)-vec2(.5,.7);
	float tl = max(max(tp.x,tp.y), 0.)*2.;
	vec3 outer = vec3(83,146,1)/255.,
		pulp = vec3(226,243,191)/255.,
		inner = vec3(139,178,0)/255.;
	inner = mix(inner, vec3(1), tl*tl);
	vec3 col = mix(inner, pulp, max(clamp((cl-.25)*50.,0.,1.), max(0.,cos(SEED.y*.3872783+atan(p.x,p.y)*10.)-.5)*2.));
	col = mix(col, outer, clamp((cl-.28)*70.,0.,1.));
	return OUTLINE(col, d);
}


vec4 Kiwi(vec2 p) {
	if (length(p) > .4) return vec4(0);
	
	p.x *= .85;
	float d = length(p)-.3;
	
	p *= vec2(.7+mod(SEED.y,.3),1.3);
	float ang = atan(p.y,p.x),
		len = length(p)+Tri(SEED.x*.3333+(1.+ang/PI)*floor(3.+mod(SEED.x*.38274,5.)))*.01;
	
	ang = mod(ang+PI*2.+SEED.x*.3987234,PI/15.)-PI/15.*.5;
	vec2 apos = vec2(sin(ang),cos(ang))*len;
	float lines = min(.4+12.*abs(apos.x)+(d+.3)*(2.+mod(SEED.y*.493,.5)), 1.);
	
	vec3 col = mix(vec3(98,157,54)/255.*lines, vec3(245,245,191)/255., clamp((.1-len)*30.,0.,1.));
	
	//seeds
	apos.y = abs(apos.y-.11)-.015;
	apos.x *= 1.6;
	col = mix(col, vec3(.01), clamp((1.5-length(apos)*100.),0.,1.));
	
	return OUTLINE(col, d);
}


vec4 Apple(vec2 p) {
	if (length(p) > .6) return vec4(0);
	
	p += cos(SEED.yx*.02+p.yx*(5.+mod(SEED*.057648,10.)))*.005;

	//leaf/stem
	vec4 lc;
	if (fract(SEED.x*.287685) < .1) {
		//leaf
		vec2 lp = p-vec2(0,.35);
		mat2 rt = Rot(mod(SEED.x*.818934,2.)-1.);
		lp = lp*rt;
		lp.y -= .05;
		lp.x += sign(lp.x)*(.03+mod(SEED.y*.007,.03));
		float ld = length(lp)-.08;
		lc = OUTLINE(vec3(90,147,33)/255.*(2.-abs(lp.x)*15.), ld);
	
	} else {
		//stem
		vec2 sp = p-vec2(0,.4);
		sp *= Rot(max(0.,sp.y+.05)*(mod(SEED.y*.376895, 20.)-10.));
		sp = abs(sp)-vec2(.005+max(0.,sp.y*.15),.05);
		float sd = max(sp.x,sp.y);
		lc = OUTLINE(vec3(131,103,70)/255., sd);
	}
	
	vec2 op = p;
	
	//apple
	p.x *= (1.-p.y*.4);
	p.y -= p.x*p.x*0.5*sign(p.y);
	vec2 ap = abs(p)-vec2(.15,.16);
	float d = length(max(ap,0.))+min(0.,max(ap.x,ap.y))-.2;
	
	vec2 sp = op, soffset = mod(SEED*PI,100.);
	sp = (sp+2.+soffset+cos(sp.yx*16.)*.02)/.015;
	vec2 fp = floor(sp/2.);
	sp = mod(sp,2.)-1.;
	vec3 acol = vec3(195,8,11)/255.,
		yellow = mix(vec3(198,213,117)/255., vec3(149,182,39)/255., fract(SEED.x*.333333)),
		light = vec3(229,96,80)/255.;
	
	if ((fract(dot(fp,vec2(.327865,.537264)))*.5+fract(dot(fp,vec2(.1327,.28735)))*.5) < .4-op.y*.4) {
		acol = mix(acol, light,  fract(SEED.y*.575)*clamp(1.1-length(sp)*2., 0.,1.));
	}
	
	op += soffset;
	acol = mix(acol,yellow, (cos(dot(op,vec2(2,6))+cos(op.x*8.)*mod(SEED.x*1.327645,2.))*.5+.5)*fract(SEED.y*.7386345));
	
	vec4 ac = OUTLINE(acol, d);
	
	return BLEND(ac,lc);
}


vec4 Banana(vec2 p) {
	if (length(p) > .85) return vec4(0);
	
	p += cos(SEED.yx*.01+p.yx*(5.+mod(SEED*.0838764,10.)))*.005;
	
	//stem
	vec2 sp = p-vec2(.4,.59);
	sp *= Rot(-1.3+mod(SEED.x*.674,.4));
	sp = abs(sp)-vec2(.02,.06);
	float sd = max(sp.x,sp.y);
	
	//fruit
	float bpy = p.y;
	p.x -= .85*p.y*p.y;
	p.y = p.y-clamp(p.y,-.45,.45);
	float d = SmoothMin(length(p)-.15, sd, .02);
	
	
	vec3 col = vec3(236,205,20)/255.;
	float lx = mod((p.x+2.+mod(SEED.y*.1365,2.))*(6.+mod(SEED.x*.432,8.)),2.)-1.;
	col = mix(col, vec3(244,233,115)/255., abs(lx));
	
	//green top stem
	col = mix(col, vec3(165,175,24)/255., clamp((bpy+p.x*.55-.7+mod(SEED.x*.033333,.2))*10.,0.,1.));

		
	return OUTLINE(col, d);
}


vec4 Mango(vec2 p) {
	if (length(p) > .6) return vec4(0);
	
	p += cos(SEED.yx*.01+p.yx*(5.+mod(SEED*.0838764,10.)))*.007;
	
	vec2 sp = p;
	
	p.x *= 1.2-0.3*p.y*p.y+p.y*.2;
	float d = length(p)-.5;
	
	vec2 soffset = mod(SEED*PI,100.);
	sp = (sp+2.+soffset)/.008;
	vec2 fp = floor(sp/2.);
	sp = mod(sp,2.)-1.;
	
	float speck = 0.;	
	if ((fract(dot(fp,vec2(.7624,.3238)))*.5+fract(dot(fp,vec2(.1327,.28735)))*.5) < .4) {
		float sgn = fract(dot(fp,vec2(.67364,.42832)))<.5 ? -1. : 1.;
		speck = clamp(1.1-length(sp),0.,1.)*.2*sgn;
	}
	
	float td = length(p-(mod(SEED*.388,1.)-.5));
	vec3 col = vec3(240,176,16)/255.;
	col = mix(col, vec3(152,157,31)/255., fract(SEED.x*.234));
	col = mix(col, vec3(224,71,40)/255., pow(clamp(td+speck,0.,1.), 2.));
	return OUTLINE(col, d);
}


vec4 Watermelon(vec2 p) {
	if (length(p) > 1.6) return vec4(0);
	
	const vec2 dir = normalize(vec2(-.5,1.));
	float cl = length(p),
		d = max(cl-1.5,
	max(dot(p,dir),dot(p,vec2(dir.x,-dir.y))));
	
	float ang = atan(p.x,p.y);
	vec2 tp = vec2(ang/PI,cl*.1);
	tp = fract((tp+10.+cos(tp.yx*100.+4.*fract(SEED*.1278643))*.01)*200.)*2.-1.;
	tp = abs(tp)-vec2(.6,.7);
	float tl = max(max(tp.x,tp.y), 0.)*2.;
	
	vec2 ind = mod(SEED*.3786542, vec2(.01,.06));
	vec3 outer = vec3(72,117,32)/255.,
		pulp = vec3(254,122,123)/255.;
	outer =  mix(vec3(205,219,166)/255., outer, clamp((cl-1.46+ind.x)*27.,0.,1.));
	pulp = mix(pulp, vec3(222,44,40)/255., clamp(cl-.3-tl*.4,0.,1.));
	outer = mix(pulp, outer, clamp((cl-1.35+ind.y)*12.,0.,1.));
	
	//seeds
	ang = mod(ang+PI*2.+SEED.x*.3987234,PI/10.)-PI/10.*.5;
	vec2 apos = vec2(sin(ang),cos(ang))*cl;
	apos.y -= .6+mod(SEED.x*.877,.15);
	apos *= vec2(1.5,.5);
	apos.x = abs(apos.x)-.04;
	outer = mix(outer, vec3(28,11,19)/255., clamp((4.-length(apos)*200.),0.,1.));
	
	return OUTLINE(outer, d);
}


vec4 Pineapple(vec2 p) {
	if (length(p) > 1.) return vec4(0);
	
	float ang = atan(p.x,p.y),
		cl = length(p),
		d = cl-.9+Tri((ang/PI+1.)*(1.+floor(mod(SEED.x*.75345,4.)))+SEED.y*.43)*.05;
		
	vec2 dsz = .75+mod(SEED*.12785,vec2(1.5));
	vec3 col = vec3(247,226,101)/255.;
	col = mix(col, vec3(255,235,146)/255., cos(SEED.x*.57231+ang*10.+cos(SEED.x+ang*4.+cl*10.)*dsz.x+cos(SEED.y+ang*24.)*dsz.y)*.5+.5);
	col = mix(vec3(236,201,55)/255., col, clamp((cl-.15+mod(SEED.x*.7923,.05))*12.,0.,1.));
	
	return OUTLINE(col, d);
}


void main() {
	vec2 pos = uv;
	vec4 c = vec4(0);
	
	//fruits
	c += Blueberry(pos-vec2(.3,.3));
	
	c += Raspberry(pos-vec2(.9,.35));
	
	c += Strawberry(pos-vec2(1.6,.35));
	
	c += Lime(pos-vec2(2.5,.45));
	
	c += Kiwi(pos-vec2(3.45,.44));
	
	c += Apple(pos-vec2(.6,1.2));
	
	c += Banana(pos-vec2(2.2,1.3));
	
	c += Mango(pos-vec2(3.4,1.6));
	
	c += Watermelon(pos-vec2(2.3,3.15));
	
	c += Pineapple(pos-vec2(1.06,2.94));
	
	gl_FragColor = c;
}

</script>



<script type="text/plain" id="SpriteV.glsl">
//sprite vertex shader
attribute vec4 vPos;
varying vec2 uv;

void main() {
	uv = vPos.zw;
	gl_Position = vec4(vPos.xy,0,1);
}
</script>

<script type="text/plain" id="Sprite.glsl">
//sprite frag shader
varying vec2 uv;
uniform sampler2D tex;

void main() {
	gl_FragColor = texture2D(tex,uv);
}
</script>

<script type="text/javascript">
