//Ethan Alexander Shulman 2023
/*File: System.js

*/


/*Function: LoadBinary
Load binary data from URL.

Parameters:
*string* url - URL.
*function(data)* onLoad - Function called when loading complete, 'data' is an *ArrayBuffer* of the binary data or null if failed.*/
function LoadBinary(url,onLoad) {
	var req = new XMLHttpRequest();
	req.onLoadBinaryCallback = onLoad;
	req.onreadystatechange = OnLoadBinaryStateChanged;
	req.responseType = "arraybuffer";
	req.open("GET",url);
	req.send();
}
//HTTP request callback for LoadBinary.
function OnLoadBinaryStateChanged(e) {
	var r = e.target;
	if (r.readyState === 4) {
		r.onLoadBinaryCallback((r.status>=200&&r.status<=299)?r.response:null);
	}
}


/*Function: LoadText

Parameters:
**string* eleId - Plain text element id.

Returns:
*string* Text from element.*/
function LoadText(eleId) {
	return document.getElementById(eleId).textContent;
}


/*Function: Array.prototype.fastRemove
Quickly remove a single element at index i, but breaks order of array by moving end element to removed element.
Does not perform error checking, if index is out of bounds undefined behaviour will occur.

Parameters:
*int* i - Index of element to remove in array.*/
Array.prototype.fastRemove = function(i) {
	var ei = this.length-1;
	if (i !== ei) this[i] = this[ei];
	this.length = ei;
}



/*Class: ObjectPool
Object pool, inherits from Array.

Constructor: ObjectPool
Constructs new ObjectPool.

Parameters:
*function* objClass - Object class constructor.*/
/**@constructor*/
function ObjectPool(objClass) {
	Array.call(this);
	this.objectClass = objClass;
}
ObjectPool.prototype = Object.create(Array.prototype);
/*Function: Get
Get object from pool.

Returns:
*Object**/
ObjectPool.prototype.Get = function() {
	var l = this.length;
	if (l) {
		l--;
		var o = this[l];
		this.length = l;
		return o;
	} else {
		return new this.objectClass();
	}
}
/*Function: Return
Return object to pool.

Paramters:
*Object* o*/
ObjectPool.prototype.Return = function(o) {
	this.push(o);
}
